#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/bsd_types.h>
#include <sys/socket.h>
#include <malloc.h>
#include "confapi.h"
#include "confmgr.h"
#include "mgr.h"
/*#include "error.h" */

/* #define MESGLEN 4 */

conference *register_application(int appType, char *CNAME)
{
  int n, msglen;
  char* mesg;   
  struct sockaddr_in conf_mgr_addr;
  
  /* allocate memory on heap */  
  conference *app = (conference *) malloc((size_t)
					  sizeof(conference));  
  if (!app)
    err_dump("register_application: memory allocation failure");
  
  msglen = sizeof(char) * (3 + strlen(CNAME));
  mesg = (char*) malloc(msglen);
  if (!mesg) {
     fprintf(stdout, "register: memory allocation failure\n");
     exit(1);
  }
  mesg[0] = appType & 255;
  mesg[1] = (char) 1 + strlen(CNAME);
  mesg[2] = (char) CONFRegisterApp;
  strcpy(mesg + 3, CNAME);
    
  /* fill in the conf_mgr_addr structure */
  bzero((char *) &conf_mgr_addr, sizeof(conf_mgr_addr));
  conf_mgr_addr.sin_family      = PF_INET;
  conf_mgr_addr.sin_addr.s_addr = inet_addr(CONF_MGR_HOST_ADDR);/*loopback*/
  conf_mgr_addr.sin_port        = htons(CONF_MGR_PORT);    /* 7000 */
  
  /* open a UDP socket */
  if ( (app -> sockfd = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
    err_dump("register_application : can't open datagram socket");
  else
    fprintf(stdout, "register_application : Socket created \n");
  
  if (connect(app -> sockfd, (struct sockaddr *) & conf_mgr_addr,
	      sizeof(conf_mgr_addr)) < 0)
    err_sys("register_application : can't do 'connect' to conference
            manager ");
  else
    fprintf(stdout, "register_application: connect done \n");
  
  if (send(app -> sockfd, mesg, msglen, 0) == -1)
    err_dump("register_application: couldn't send to conference
             manager ");
  else
    fprintf(stdout, "the message sent to conference manager: %s\n",
	    mesg);
  n = recv(app -> sockfd, app -> HANDLE, sizeof(uint), 0);
  if (n == -1)
    err_sys("recv error for HANDLE \n");
  else
    fprintf(stdout, "HANDLE = %d\n", app -> HANDLE);

  app -> mcast_addr =  /* a multicast address to be assigned */
  app -> port =  /* a port address at which to send/receive messages */
  app -> late = '1';  /* for now */

  free(mesg);

  return app;
}
 

int deregister(conference *application){

  /* Contact conf mgr with deregister msg */

  /* Look at response from conf mgr. If ok then free conference
     memory and return 1. else return 0 and don't free conference
     memory.*/
  char *mesg, temp[sizeof(char)* 50];
  int n, len = 256, mesglen, returnVal, confirmation;

  sprintf(temp, "%u", application -> HANDLE); /* do this to get HANDLE len*/
  
  mesglen = strlen(temp) + 3;
  mesg = (char *) malloc(mesglen);
  if (!mesg) {
     fprintf(stdout, "memory allocation failure in deregister\n"); 
     exit(1);
  }
  
  mesg[0] = (char) (mesglen - 1);
  mesg[1] = '0';
  mesg[2] = (char) CONFDeregisterApp;
/*  strcpy(mesg+3, (char*)application -> HANDLE);  */
  strcpy(mesg+3,temp);  /* copy HANDLE as a string of chars */
 
  if (send(application -> sockfd, mesg, mesglen, 0) == -1)
     err_dump("deregister : couldn't send message");
  else
     fprintf(stdout, "deregister: sent deregister request");
 
  n =  recv(application -> sockfd, &confirmation, len, 0);
  if (n == -1)
     err_sys("recv error in deregister");
  else {
     if (confirmation == DEREGISTERED) {
        fprintf(stdout, "Received confirmation message");
        free (application);
        returnVal = 1;
     }
     else {
        returnVal = 0;
        fprintf(stdout, "deregister: mgr couldn't deregister
                application, check if application is registered\n");
     }
  }
  
  free(mesg);
  return returnVal;
  
}
  
